package collision;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;

import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import java.io.IOException;
import java.util.ArrayList;

import java.util.logging.Level;
import java.util.logging.Logger;
import javax.sound.midi.InvalidMidiDataException;
import javax.sound.midi.MidiSystem;
import javax.sound.midi.MidiUnavailableException;
import javax.sound.midi.Sequence;
import javax.sound.midi.Sequencer;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.Clip;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.UnsupportedAudioFileException;
import javax.swing.JPanel;
import javax.swing.Timer;

/**
 *
 * @author edu
 */
public class Board extends JPanel implements ActionListener {

    private int velocidadeAliens;
    private Timer timer;
    private Jogador jogador;
    private ListaAlien alien;
    private boolean ingame;
    private int B_WIDTH;
    private int B_HEIGHT;
    private Clip somAlarme;

    /**
     *
     * @param indPersonagem 0 -
     * Este é o modulo principal do jogo
     * são criados os aliens e o jogador.
     * são testados as colisões dos misseis, aliens e jogador.
     * é capturado os eventos do teclado.
     * o timer é responsavel por mover os objetos na tela e verificar as colisões
     */
    public Board(int indPersonagem) {

// cria o personagem
        switch (indPersonagem) {
            case 0:
                jogador = new Duke();
                break;
            case 1:
                jogador = new Puffy();
                break;
            case 2:
                jogador = new Tux();
                break;
            default:
                return;
        }

// configura para capturar eventos do teclado
        addKeyListener(new TAdapter());
//        addMouseMotionListener(new TMouseBoard());
//        addMouseListener(new TMouseBoardPress());

        setFocusable(true);
        setBackground(Color.white);
        setDoubleBuffered(true);

        ingame = true;

        setSize(400, 300);

// velocidade inicial dos aliens
        velocidadeAliens = 1;
// cria os primeiros aliens do jogo
        alien = new ListaAlien(velocidadeAliens);
// música de fundo
        tocaMid("/collision/mortal_kombat.mid",50000);
// base de tempo do jogo
        timer = new Timer(16, this);
        timer.start();
    }

    @Override
    public void addNotify() {
        super.addNotify();
        B_WIDTH = getWidth();
        B_HEIGHT = getHeight();
    }

    @Override
    public void paint(Graphics g) {
        super.paint(g);

        if (ingame) {

            Graphics2D g2d = (Graphics2D) g;
            g2d.setColor(Color.WHITE);

            if (jogador.isVisivel()) {
                if (jogador.getEscudo() > 0) {
                    g2d.setColor(Color.RED);
                    g2d.draw(jogador.getArea());
                    g2d.drawString("" + jogador.getEscudo(), jogador.getPosX(), jogador.getPosY());
                    g2d.setColor(Color.WHITE);
                }
                g2d.drawImage(jogador.getFigura(), jogador.getPosX(), jogador.getPosY(),
                        this);
            }

            ArrayList ms = jogador.getMissil();

            for (int i = 0; i < ms.size(); i++) {
                Missile m = (Missile) ms.get(i);
                g2d.drawImage(m.getImage(), m.getX(), m.getY(), this);
            }

            for (int i = 0; i < alien.size(); i++) {
                Alien a = alien.get(i);
                if (a.isVisivel()) {
                    g2d.drawImage(a.getFigura(), a.getPosX(), a.getPosY(), this);
                    ms = a.getMissil();
                    for (int j = 0; j < ms.size(); j++) {
                        Missile m = (Missile) ms.get(j);
                        g2d.drawImage(m.getImage(), m.getX(), m.getY(), this);
                    }

                }
            }

            g2d.setColor(Color.BLACK);

            for (int i = 0; i < alien.size(); i++) {
                Alien a = alien.get(i);
                if (a.isVisivel()) {
                    g2d.drawString("" + a.getVida(), a.getPosX() + 25, a.getPosY() + 15);
                }
            }

            g2d.drawString("vida: " + jogador.getVida(), 5, 15);
            g2d.drawString("Poder: " + jogador.getPoder(), 5, 25);
            g2d.drawString("Pontos: " + jogador.getPontos(), 5, 35);


        } else {
            String msg = "Game Over";
            Font small = new Font("Helvetica", Font.BOLD, 14);
            FontMetrics metr = this.getFontMetrics(small);

            g.setColor(Color.black);
            g.setFont(small);
            g.drawString(msg, (B_WIDTH - metr.stringWidth(msg)) / 2,
                    B_HEIGHT / 2);
        }

        Toolkit.getDefaultToolkit().sync();
        g.dispose();
    }

    public void actionPerformed(ActionEvent e) {

        jogador.setTempo(16);

        if (alien.size() == 0) {
            alien = new ListaAlien(++velocidadeAliens);
        }

        ArrayList ms = jogador.getMissil();

        for (int i = 0; i < ms.size(); i++) {
            Missile m = (Missile) ms.get(i);
            if (m.isVisible()) {
                m.move();
            } else {
                ms.remove(i);
            }
        }

        for (int i = 0; i < alien.size(); i++) {
            Alien a = alien.get(i);
            if (a.isVisivel()) {
                ms = a.getMissil();
                for (int j = 0; j < ms.size(); j++) {
                    Missile m = (Missile) ms.get(j);
                    if (m.isVisible()) {
                        m.move();
                    } else {
                        ms.remove(j);
                    }
                }

            }
        }


        jogador.move();
        alien.move();
        checkCollisions();

        if (jogador.getVida() > 10) {
            if (this.somAlarme != null) {
                this.somAlarme.stop();
                this.somAlarme = null;
            }

        }
        repaint();
    }

    public void checkCollisions() {

        /**
         * detecta colisão da nave com alien e misseis
         */
        Rectangle r3 = jogador.getArea();

        for (int j = 0; j < alien.size(); j++) {
            Alien a = (Alien) alien.get(j); //lista de aliens
            ArrayList ms = a.getMissil();   //lista de misseis do alien

            // detecta colisão da nave com alien
            if (a.isVisivel()) {
                if (r3.intersects(a.getArea())) {
                    tocaWav("/collision/COLISION.WAV", 0);
                    jogador.incPontos(1);
                    jogador.incPoder(5);
                    a.setVisivel(false); //retira alien do jogo
                    if (jogador.getEscudo() == 0) {
                        if (jogador.decrementaVida(50) == 0) //decrementa
                        {
                            ingame = false;
                        }
                    }
                    if (jogador.getVida() <= 10) {
                        if (this.somAlarme == null) {
                            this.somAlarme = tocaWav("/collision/CRASHBUZ.WAV", 100);
                        }
                    }

                }
            }

            // detecta colisão da nave com misseis
            for (int i = 0; i < ms.size(); i++) {
                Missile m = (Missile) ms.get(i);

                if (m.isVisible()) {
                    if (r3.intersects(m.getBounds())) {
                        m.setVisible(false);
                        tocaWav("/collision/COLISION.WAV", 0);
                        if (jogador.getEscudo() == 0) {
                            if (jogador.decrementaVida(10) == 0) //decrementa
                            {
                                ingame = false;
                            }
                        }
                        if (jogador.getVida() <= 10) {
                            if (this.somAlarme == null) {
                                this.somAlarme = tocaWav("/collision/CRASHBUZ.WAV", 100);
                            }
                        }
                    }
                }


            }
        }
        /**
         * detecta colisão do alien com missil
         */
        ArrayList ms = jogador.getMissil();
        for (int i = 0;
                i < ms.size();
                i++) {
            Missile m = (Missile) ms.get(i);

            Rectangle r1 = m.getBounds();

            for (int j = 0; j < alien.size(); j++) {
                Alien a = (Alien) alien.get(j);
                Rectangle r2 = a.getArea();

                if (r2.intersects(r1)) {
                    m.setVisible(false);
                    if (a.decrementaVida(10) == 0) { //decrementa vida do alien
                        jogador.incPontos(1);
                        jogador.incPoder(5);

                    }
                    tocaWav("/collision/COLISION.WAV", 0);
                }
            }
        }
    }

    private class TAdapter extends KeyAdapter {

        @Override
        public void keyReleased(KeyEvent e) {
            jogador.keyReleased(e);
        }

        @Override
        public void keyPressed(KeyEvent e) {
            switch (jogador.keyPressed(e)) {
                case 1:
                case 2:
                case 3:
                    alien.apagarAliens();
                    tocaWav("/collision/BINGBING.WAV", 0);
                    break;
            }
        }
    }

    private class TMouseBoard extends MouseMotionAdapter {

        @Override
        public void mouseMoved(MouseEvent evt) {
        }
    }

    private class TMouseBoardPress extends MouseAdapter {

        @Override
        public void mousePressed(MouseEvent evt) {
        }
    }

    public void tocaMid(String s,int x) {
        try {
            Sequence sequence = MidiSystem.getSequence(getClass().getResource(s));
            Sequencer sequencer = MidiSystem.getSequencer();
            sequencer.open();
            sequencer.setSequence(sequence);
            sequencer.setLoopCount(x);
            sequencer.start();
        } catch (MidiUnavailableException ex) {
            Logger.getLogger(Board.class.getName()).log(Level.SEVERE, null, ex);
        } catch (InvalidMidiDataException ex) {
            Logger.getLogger(Board.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
            Logger.getLogger(Board.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    public Clip tocaWav(String s, int x) {

        AudioInputStream musica = null;
        try {
            Clip clip = AudioSystem.getClip();
            musica = AudioSystem.getAudioInputStream(getClass().getResource(s));
            clip.open(musica);
            clip.loop(x);
            clip.start();
            return clip;
        } catch (UnsupportedAudioFileException ex) {
            Logger.getLogger(Board.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
            Logger.getLogger(Board.class.getName()).log(Level.SEVERE, null, ex);
        } catch (LineUnavailableException ex) {
            Logger.getLogger(Board.class.getName()).log(Level.SEVERE, null, ex);
        } finally {
            try {
                musica.close();
            } catch (IOException ex) {
                Logger.getLogger(Board.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
        return null;

    }
}

